/**
 * Classe dei quadrati colorati.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class ColoredSquare extends Square {
    /**
     * Ritorna la componente intensita' del colore del quadrato.
     *
     * @return La componente intensita'.
     */
    public double getIntensity() {
        return (red / 255.0 + green / 255.0 + blue / 255.0) / 3;
    }

    /**
     * Ritorna la componente saturazione del colore del quadrato.
     *
     * @return La componente saturazione.
     */
    public double getSaturation() {
        int min = red;

        if(green < min) min = green;
        if(blue  < min) min = blue;

        return 1 - min/getIntensity();
    }

    /**
     * Costruttore per i quadrati colorati.
     *
     * Il colore viene specificato mediante i tre parametri <code>r</code>, 
     * <code>g</code> e <code>b</code>, che sono numeri interi tra 0 e 255.
     *
     * @param s Lato del quadrato.
     * @param r Quantita' di rosso nel colore.
     * @param g Quantita' di verde nel colore.
     * @param b Quantita' di blu nel colore.
     */
    public ColoredSquare(int s, int r, int g, int b) {
        super(s);

        red = r; green = g; blue = b;
    }

    /**
     * Componente rossa del colore del quadrato.
     */
    private int red = 0;

    /**
     * Componente verde del colore del quadrato.
     */
    private int green = 0;

    /**
     * Componente blu del colore del quadrato.
     */
    private int blue = 0;
}
