package bancomat;

/**
 * Classe delle transazioni utilizzate dal sistema Bancomat.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class Transaction {
    /**
     * Costruttore dell'oggetto transazione.
     */
    public Transaction(Account a, int am) {
        account = a;
        bank    = a.getBank();
        amount  = am;
        open    = true;
    }

    /**
     * Completa le operazioni di prelievo.
     */
    public void completeGetMoney() {
        bank.completeGetMoney(this);

        open = false;
    }

    /**
     * Annulla le operazioni di prelievo.
     */
    public void rollbackGetMoney() {
        bank.rollbackGetMoney(this);

        open = false;
    }

    /**
     * Indica se la transazione e' correntemente aperta.
     */
    public boolean isOpen() {
        return open;
    }

    /**
     * Restituisce la somma relativa alla transazione.
     *
     * @return La somma relativa alla transazione.
     */
    public int getAmount() {
        return amount;
    }

    /**
     * La somma relativa alla transazione.
     */
    private int amount = 0;

    /**
     * Il conto corrente.
     */
    private Account account = null;

    /**
     * La banca.
     */
    private Bank bank = null;

    /**
     * Indica se la transazione e' ancora aperta.
     */
    private boolean open;
}
