package bancomat.test;

import bancomat.*;

/**
 * Classe delle interfacce verso la banca da utilizzare per il testing.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class TestBank implements Bank {
    /**
     * Crea l'oggetto.
     */
    public TestBank(Controller c) { 
        controller = c;
    }

    /**
     * Attiva l'oggetto.
     */
    public void run() {}

    /**
     * Valida la coppia numero di carta/PIN. Se la validazione ha 
     * successo informa il controller del conto corrente associato.
     */
    public void validate(String cardNumber, String pin) throws InvalidPINException {
        controller.logDebug("Richiesta la validazione di '" + cardNumber + "' con PIN '" + pin + "'");

        if(cardNumber == null || cardNumber.length() == 0 ||
           pin        == null || pin.length()        == 0) throw new InvalidPINException();

        // controlla i valori utilizzabili per il testing
        if(cardNumber.equals("123") && pin.equals("456")) {
            controller.logDebug("Numero di carta e PIN validati");

            controller.accountFound(new Account(this, "123456789012", "65654", "66456"));

            return;
        }

        throw new InvalidPINException();
    }

    /**
     * Autorizza l'inizio delle operazioni di prelievo.
     */
    public Transaction startGetMoney(Account a, int amount) throws InvalidTransactionException {
        controller.logDebug("Richiesto un prelievo di " + amount);

        // controlla il valore utilizzabile per il testing        
        if(amount < 50 || amount > 500) throw new InvalidTransactionException();

        controller.logDebug("Prelievo concesso");

        return new Transaction(a, amount);
    }

    /**
     * Completa le operazioni di prelievo.
     */
    public void completeGetMoney(Transaction transaction) {
        controller.logDebug("Transazione terminata con successo");
    }

    /**
     * Annulla le operazioni di prelievo.
     */
    public void rollbackGetMoney(Transaction transaction) {
        controller.logDebug("Transazione annullata");
    }

    /**
     * Il controller utilizzato dal sistema.
     */
    private Controller controller = null;
}
