package bancomat.test;

import bancomat.*;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;

/**
 * Classe delle interfacce utente utilizzate per il testing.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class TestGUI extends JFrame implements GUI {
    /**
     * Crea l'oggetto.
     */
    public TestGUI(Controller c) { 
        controller = c;
        screen     = new JPanel();

        screen.setLayout(new BorderLayout());
    }

    /**
     * Attiva l'oggetto.
     */
    public void run() {
        timeout = new JButton("Timeout");
        timeout.setForeground(Color.red);

        timeout.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.userTimeoutExpired();
                }
            }
        );

        exit = new JButton("Esci");
        exit.setForeground(Color.green);

        exit.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.forcedExit();
                }
            }
        );

        getContentPane().add(screen);

        setTitle("Bancomat - Modalita' Test");
        setSize(400, 400);

        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        showInitialScreen();

        setVisible(true);

        controller.logDebug("GUI di test visualizzata");
    }

    /**
     * Metodo che visualizza la schermata iniziale.
     */
    public void showInitialScreen() {
        JLabel  text = new JLabel("Schermata iniziale", SwingConstants.CENTER);
        JButton ok   = new JButton("OK");

        // per rendere questo componente accessibile dall'action listener 
        // definito nel seguito, mettiamo la clausola 'final'
        final JTextField cardNumber = new JTextField("");

        // diamo una dimensione preferita al campo di testo
        // in modo da visualizzarlo correttamente
        // l'altezza del campo di testo e' quella dei bottoni che ha accanto
        Dimension d = ok.getPreferredSize();
        d.setSize(80, d.getHeight());
        cardNumber.setPreferredSize(d);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);
        buttonPanel.add(cardNumber);
        buttonPanel.add(ok);

        exit.setEnabled(false); timeout.setEnabled(false);

        JPanel initialScreen = new JPanel();
        initialScreen.setLayout(new BorderLayout());
        initialScreen.add(text, BorderLayout.CENTER);
        initialScreen.add(buttonPanel, BorderLayout.SOUTH);

        ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.cardNumberEntered(cardNumber.getText());
                }
            }
        );

        showScreen(initialScreen);

        controller.logDebug("Visualizzata schermata iniziale");
    }

    /**
     * Metodo che visualizza la schermata d'errore.
     */
    public void showErrorScreen(String message) {
        JLabel  text    = new JLabel("Errore: " + message, SwingConstants.CENTER);
        JButton timeout = new JButton("Timeout");

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);

        exit.setEnabled(false); timeout.setEnabled(true);

        JPanel errorScreen = new JPanel();
        errorScreen.setLayout(new BorderLayout());
        errorScreen.add(text, BorderLayout.CENTER);
        errorScreen.add(buttonPanel, BorderLayout.SOUTH);

        timeout.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.reset();
                }
            }
        );

        showScreen(errorScreen);

        controller.logDebug("Visualizzata schermata di errore");
    }

    /**
     * Metodo utilizzato per espellere la carta.
     */
    public void ejectCard() {
        JOptionPane.showMessageDialog(this, "Carta espulsa");

        controller.logDebug("Carta espulsa");
    }

    /**
     * Metodo utilizzato per ritirare la carta.
     */
    public void retireCard() {
        JOptionPane.showMessageDialog(this, "Carta ritirata");

        controller.logDebug("Carta ritirata");
    }

    /**
     * Metodo che visualizza la schermata di richiesta del PIN.
     */
    public void showRequestPINScreen() {
        JLabel  text = new JLabel("Introdurre il proprio PIN", SwingConstants.CENTER);
        JButton ok   = new JButton("OK");

        // per rendere questo componente accessibile dall'action listener 
        // definito nel seguito, mettiamo la clausola 'final'
        final JTextField pin = new JTextField("");

        // diamo una dimensione preferita al campo di testo
        // in modo da visualizzarlo correttamente
        // l'altezza del campo di testo e' quella dei bottoni che ha accanto
        Dimension d = ok.getPreferredSize();
        d.setSize(80, d.getHeight());
        pin.setPreferredSize(d);
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);
        buttonPanel.add(pin);
        buttonPanel.add(ok);

        exit.setEnabled(true); timeout.setEnabled(true);

        JPanel requestPINScreen = new JPanel();
        requestPINScreen.setLayout(new BorderLayout());
        requestPINScreen.add(text, BorderLayout.CENTER);
        requestPINScreen.add(buttonPanel, BorderLayout.SOUTH);

        ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.pinEntered(pin.getText());
                }
            }
        );

        showScreen(requestPINScreen);

        controller.logDebug("Visualizzata schermata di richiesta PIN");
    }

    /**
     * Metodo che visualizza la schermata contenente il menu' principale.
     */
    public void showMainMenuScreen() {
        JLabel  text         = new JLabel("Scegliere l'operazione da effettuare", SwingConstants.CENTER);
        JButton movementList = new JButton("Estratto Conto");
        JButton getMoney     = new JButton("Prelievo");
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);
        buttonPanel.add(getMoney);
        buttonPanel.add(movementList);

        exit.setEnabled(true); timeout.setEnabled(true);

        JPanel mainMenuScreen = new JPanel();
        mainMenuScreen.setLayout(new BorderLayout());
        mainMenuScreen.add(text, BorderLayout.CENTER);
        mainMenuScreen.add(buttonPanel, BorderLayout.SOUTH);

        movementList.setEnabled(false);

        getMoney.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.getMoneyChosen();
                }
            }
        );

        movementList.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    // per ora il bottone e' disabilitato e non si dovrebbe arrivarre mai qui
                }
            }
        );

        showScreen(mainMenuScreen);

        controller.logDebug("Visualizzato menu' principale");
    }

    /**
     * Metodo utilizzato per erogare una somma.
     */
    public void provideMoney(int amount) {
        JLabel  text = new JLabel("Somma erogata: " + amount, SwingConstants.CENTER);
        JButton ok   = new JButton("Ritira somma e carta");
        
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);
        buttonPanel.add(ok);

        exit.setEnabled(false); timeout.setEnabled(true);

        JPanel provideMoneyScreen = new JPanel();
        provideMoneyScreen.setLayout(new BorderLayout());
        provideMoneyScreen.add(text, BorderLayout.CENTER);
        provideMoneyScreen.add(buttonPanel, BorderLayout.SOUTH);

        ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.getMoneyCompleted();
                }
            }
        );

        showScreen(provideMoneyScreen);

        controller.logDebug("Erogata somma: " + amount);
    }

    /**
     * Metodo che visualizza la schermata di richiesta della somma da prelevare.
     */
    public void showRequestMoneyScreen() {
        JLabel  text = new JLabel("Introdurre la somma che si intende prelavare", SwingConstants.CENTER);
        JButton ok   = new JButton("OK");

        // per rendere questo componente accessibile dall'action listener 
        // definito nel seguito, mettiamo la clausola 'final'
        final JTextField number = new JTextField("");

        // diamo una dimensione preferita al campo di testo
        // in modo da visualizzarlo correttamente
        // l'altezza del campo di testo e' quella dei bottoni che ha accanto
        Dimension d = ok.getPreferredSize();
        d.setSize(80, d.getHeight());
        number.setPreferredSize(d);

        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new FlowLayout());
        buttonPanel.add(exit);
        buttonPanel.add(timeout);
        buttonPanel.add(number);
        buttonPanel.add(ok);

        exit.setEnabled(true); timeout.setEnabled(true);

        JPanel requestMoneyScreen = new JPanel();
        requestMoneyScreen.setLayout(new BorderLayout());
        requestMoneyScreen.add(text, BorderLayout.CENTER);
        requestMoneyScreen.add(buttonPanel, BorderLayout.SOUTH);

        ok.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent event) {
                    controller.amountEntered(number.getText());
                }
            }
        );

        showScreen(requestMoneyScreen);

        controller.logDebug("Visualizzata schermata di richiesta somma");
    }

    /**
     * Metodo utilizzato per visualizzare una schermata.
     */
    private void showScreen(JPanel newScreen) {
        // rimuovo tutto dal JPanel che contiene le schermate
        screen.removeAll();

        // Aggiungo la nuova schermata
        screen.add(newScreen, BorderLayout.CENTER);

        // forzo Swing a ridisegnare il JPanel che contiene le schermate
        screen.validate();
    }

    /**
     * JButton utilizzato per forzare l'uscita.
     */
    private JButton exit = null;

    /**
     * JButton utilizzato per simulare i timeout dell'utente.
     */
    private JButton timeout = null;

    /**
     * Il controller utilizzato dal sistema.
     */
    private Controller controller = null;

    /**
     * Il JPanel utilizzato per contenere le schermate.
     */
    private JPanel screen = null;
}
