import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

/**
 * Classe dei controller dell'editor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class EditorController implements MouseListener {
    /**
     * Costruttore del controller dell'editor.
     *
     * @param m Il modello dell'editor.
     * @param v La vista dell'editor.
     */
    public EditorController(EditorModel m, EditorView v) {
        model = m;
        view  = v;

        view.addMouseListener(this);
    }

    /**
     * Metodo invocato quando viene fatto click sul mouse.
     *
     * @param e Evento.
     */
    public void mouseClicked(MouseEvent e) {}

    /**
     * Metodo invocato quando il puntatore del mouse entra nel componente ascoltato.
     *
     * @param e Evento.
     */
    public void mouseEntered(MouseEvent e) {}

    /**
     * Metodo invocato quando il puntatore del mouse esce dal componente ascoltato.
     *
     * @param e Evento.
     */
    public void mouseExited(MouseEvent e) {}

    /**
     * Metodo invocato quando viene premuto un bottone mouse.
     *
     * @param e Evento.
     */
    public void mousePressed(MouseEvent e) {
        int x = e.getX(), y = e.getY();

        for(int i = model.getSize() - 1; i >= 0; i--) {
            ViewablePolygon vp = model.getViewablePolygon(i);

            if(vp.contains(x, y)) {
                if(selected != vp) changeSelected(vp);

                return;
            }
        }

        if(selected != null) changeSelected(null);

        return;
    }

    /**
     * Metodo invocato quando viene rilasciato un bottone mouse.
     *
     * @param e Evento.
     */
    public void mouseReleased(MouseEvent e) {}

    /**
     * Cambia il poligono correntemente selezionato.
     *
     * @param vp Poligono da selezionare.
     */
    private void changeSelected(ViewablePolygon vp) {
        if(vp != null)       vp.setSelected(true);
        if(selected != null) selected.setSelected(false); 

        selected = vp; 

        view.repaint();
    }

    /**
     * Modello da controllare.
     */
    private EditorModel model = null;

    /**
     * Vista da controllare.
     */
    private EditorView view = null;

    /**
     * Poligono correntemente selezionato.
     */
    private ViewablePolygon selected = null;
}
