import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

/**
 * Classe dei controller dell'editor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class EditorController implements MouseListener, MouseMotionListener {
    /**
     * Costruttore del controller dell'editor.
     *
     * @param m Il modello dell'editor.
     * @param v La vista dell'editor.
     */
    public EditorController(EditorModel m, EditorView v) {
        model = m;
        view  = v;

        view.addMouseListener(this);
        view.addMouseMotionListener(this);
    }

    /**
     * Metodo invocato quando viene fatto click sul mouse.
     *
     * @param e Evento.
     */
    public void mouseClicked(MouseEvent e) {}

    /**
     * Metodo invocato quando il puntatore del mouse entra nel componente ascoltato.
     *
     * @param e Evento.
     */
    public void mouseEntered(MouseEvent e) {}

    /**
     * Metodo invocato quando il puntatore del mouse esce dal componente ascoltato.
     *
     * @param e Evento.
     */
    public void mouseExited(MouseEvent e) {}

    /**
     * Metodo invocato quando viene premuto un bottone mouse.
     *
     * @param e Evento.
     */
    public void mousePressed(MouseEvent e) {
        int x = e.getX(), y = e.getY();

        for(int i = model.getSize() - 1; i >= 0; i--) {
            ViewablePolygon vp = model.getViewablePolygon(i);

            if(vp.contains(x, y)) {
                if(selected != vp) changeSelected(vp);

                return;
            }
        }

        if(selected != null) changeSelected(null);

        return;
    }

    /**
     * Metodo invocato quando viene rilasciato un bottone mouse.
     *
     * @param e Evento.
     */
    public void mouseReleased(MouseEvent e) {
        if(isDragging) { 
            isDragging = false; 

            view.setDragging(false); view.repaint();
        }
    }

    /**
     * Metodo invocato quando il puntatore del mouse viene mosso.
     *
     * @param e Evento.
     */
    public void mouseMoved(MouseEvent e) {}

    /**
     * Metodo invocato quando il puntatore del mouse viene mosso con il bottone e' premuto.
     *
     * @param e Evento.
     */
    public void mouseDragged(MouseEvent e) {
        if(selected == null) return;

        Polygon p = selected.getPolygon();

        int mx = e.getX(), my = e.getY(), 
            px = p.getX(), py = p.getY();

        if(!isDragging) { 
            lastX = mx; lastY = my;

            isDragging = true; view.setDragging(true);
        }

        int dx = lastX - px,  dy = lastY - py;

        p.setX(mx - dx);
        p.setY(my - dy);

        lastX = mx; lastY = my;

        view.repaint();
    }

    /**
     * Ritorna il poligono correntemente selezionato.
     *
     * @return Il poligono correntemente selezionato.
     */
    public ViewablePolygon getSelected() {
        return selected; 
    }

    /**
     * Ritorna lo stato di dragging del controller.
     *
     * @return Lo stato di dragging del controller.
     */
    public boolean isDragging() {
        return isDragging; 
    }

    /**
     * Cambia il poligono correntemente selezionato.
     *
     * @param vp Poligono da selezionare.
     */
    private void changeSelected(ViewablePolygon vp) {
        if(vp != null)       vp.setSelected(true);
        if(selected != null) selected.setSelected(false); 

        selected = vp; 

        view.setSelected(vp);

        view.repaint();
    }

    /**
     * Modello da controllare.
     */
    private EditorModel model = null;

    /**
     * Vista da controllare.
     */
    private EditorView view = null;

    /**
     * Poligono correntemente selezionato.
     */
    private ViewablePolygon selected = null;

    /**
     * Indica se l'editor e' in modalita' dragging.
     */
    private boolean isDragging = false;

    /**
     * Ultima coordinata X del puntatore del mouse durante il dragging.
     */
    private int lastX = -1;

    /**
     * Ultima coordinata Y del puntatore del mouse durante il dragging.
     */
    private int lastY = -1;
}
