package editor;

import editor.model.*;
import editor.view.*;

import javax.swing.JFrame;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

/**
 * Classe degli editor grafici.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class Editor extends JFrame {
    /**
     * Costruttore dell'editor grafico.
     */
    public Editor() {
        super("Simple Polygon Editor");

        model      = new EditorModel();
        view       = new EditorView(model);
        controller = new EditorController(model, view);

        setSize(500, 500);

        view.setPreferredSize(new Dimension(3000, 3000));

        Container contentPane = getContentPane();

        contentPane.setLayout(new BorderLayout());
        contentPane.add(new EditorToolBar(this), BorderLayout.NORTH);
        contentPane.add(new JScrollPane(view),   BorderLayout.CENTER);
    }

    /**
     * Aggiunge un rettangolo al modello e rinfresca la vista.
     */
    public void addRectangle() {
        Rectangle     r = new ConcreteRectangle(10, 10, 80, 100);
        RectangleView v = new ColorRectangleView(r);

        model.addViewablePolygon(new ViewableRectangle(v));

        view.repaint();
    }

    /**
     * Aggiunge un quadrato al modello e rinfresca la vista.
     */
    public void addSquare() {
        Square     s = new ConcreteSquare(200, 200, 30);
        SquareView v = new ColorSquareView(s);

        model.addViewablePolygon(new ViewableSquare(v));

        view.repaint();
    }

    /**
     * Modello dei dati manipolati dall'editor.
     */
    private EditorModel model = null;

    /**
     * Vista dei dati manipolati dall'editor.
     */
    private EditorView view = null;

    /**
     * Controllore dei dati manipolati dall'editor.
     */
    private EditorController controller = null;
}
