package editor.model;

import editor.view.*;

/**
 * Classe dei rettangoli disegnabili.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class ViewableRectangle implements ViewablePolygon {
    /**
     * Costruttore per i rettangoli disegnabili.
     *
     * @param p Il rettangolo.
     * @param v La vista del poligono.
     */
    public ViewableRectangle(RectangleView v) {
        rectangle = (Rectangle)v.getPolygon();
        view      = v;
    }

    /**
     * Testa se il punto <code>x</code>, <code>y</code> e' contenuto nel rettangolo.
     */
    public boolean contains(int x, int y) {
        return view.contains(x, y);
    }

    /**
     * Ritorna la vista del rettangolo.
     */
    public PolygonView getPolygonView() {
        return view;
    }

    /**
     * Ritorna il rettangolo.
     */
    public Polygon getPolygon() {
        return rectangle;
    }

    /**
     * Imposta la condizione di selezione.
     *
     * @param s Il valore della condizione di selezione.
     */
    public void setSelected(boolean s) {
        view.setSelected(s);
    }

    /**
     * Ritorna la condizione di selezione.
     *
     * @return Il valore della condizione di selezione.
     */
    public boolean getSelected(boolean s) {
        return view.getSelected();
    }

    /**
     * Vista del rettangolo.
     */
    private RectangleView view = null;

    /**
     * Il rettangolo.
     */
    private Rectangle rectangle = null;
}
