package editor.view;

import editor.model.*;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Classe che implementa le viste dei rettangoli.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class ColorRectangleView implements RectangleView {
    /**
     * Costruttore delle viste colorate dei rettangoli.
     *
     * @param Il rettangolo da disegnare.
     */
    public ColorRectangleView(Rectangle r) {
        rectangle     = r;
        color         = new Color(0, 0, 255);
        selectedColor = new Color(255, 0, 0);
    }

    /**
     * Disegna un rettangolo sul contesto grafico <code>g</code>.
     *
     * @param Il contesto grafico in cui disegnare.
     */
    public void paintPolygon(Graphics g) {
        g.setColor(color);

        g.fillRect(rectangle.getX(), rectangle.getY(), rectangle.getWidth(), rectangle.getHeight());

        if(selected) {
            g.setColor(selectedColor);

            g.drawRect(rectangle.getX(), rectangle.getY(), rectangle.getWidth() - 1, rectangle.getHeight() - 1);
        }
    }

    /**
     * Ritorna il poligono associato alla vista.
     *
     * @return Il poligono associato alla vista.
     */
    public Polygon getPolygon() {
        return rectangle;
    }

    /**
     * Testa se il punto <code>x</code>, <code>y</code> e' contenuto nel rettangolo.
     */
    public boolean contains(int x, int y) {
        return rectangle.getX() < x && (rectangle.getX() + rectangle.getWidth())  > x &&
               rectangle.getY() < y && (rectangle.getY() + rectangle.getHeight()) > y;
    }

    /**
     * Imposta la condizione di selezione.
     *
     * @param s Il valore della condizione di selezione.
     */
    public void setSelected(boolean s) {
        selected = s;
    }

    /**
     * Ritorna la condizione di selezione.
     *
     * @return Il valore della condizione di selezione.
     */
    public boolean getSelected() {
        return selected;
    }

    /**
     * Ritorna il colore della vista.
     *
     * @return Il colore della vista.
     */
    public Color getColor() {
        return color;
    }

    /**
     * Imposta il colore della vista.
     *
     * @param c Il colore della vista.
     */
    public void setColor(Color c) {
        color = c;
    }

    /**
     * Il rettangolo associato alla vista.
     */
    public Rectangle rectangle = null;

    /**
     * Il colore della vista.
     */
    private Color color = null;

    /**
     * Il colore del bordo della vista selezionata.
     */
    private Color selectedColor = null;

    /**
     * Indica se il rettangolo e' l'oggetto selezionato.
     */
    private boolean selected = false;
}
