package editor;

import editor.model.*;
import editor.view.*;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;

/**
 * Classe degli editor grafici.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class Editor extends JFrame {
    /**
     * Costruttore dell'editor grafico.
     */
    public Editor() {
        super("Simple Polygon Editor");

        model      = new EditorModel();
        view       = new EditorView(model);
        controller = new EditorController(this);

        toolBar   = new EditorToolBar(this);
        statusBar = new JLabel();

        setSize(500, 500);

        view.setPreferredSize(new Dimension(3000, 3000));

        Container contentPane = getContentPane();

        contentPane.setLayout(new BorderLayout());
        contentPane.add(toolBar,               BorderLayout.NORTH);
        contentPane.add(new JScrollPane(view), BorderLayout.CENTER);
        contentPane.add(statusBar,             BorderLayout.SOUTH);

        newItem    = new JMenuItem("New",        new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/New16.gif")));
        openItem   = new JMenuItem("Open...",    new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Open16.gif")));
        saveItem   = new JMenuItem("Save",       new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Save16.gif")));
        saveAsItem = new JMenuItem("Save As...", new ImageIcon(getClass().getResource("/spacer16.gif")));
        exitItem   = new JMenuItem("Exit",       new ImageIcon(getClass().getResource("/spacer16.gif")));

        newItem.setEnabled(false);    openItem.setEnabled(false); saveItem.setEnabled(false);
        saveAsItem.setEnabled(false); exitItem.setEnabled(false);

        JMenu fileMenu = new JMenu("File");
        fileMenu.add(newItem);    fileMenu.add(openItem); fileMenu.add(saveItem);
        fileMenu.add(saveAsItem); fileMenu.add(exitItem);

        cutItem   = new JMenuItem("Cut",   new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Cut16.gif")));
        copyItem  = new JMenuItem("Copy",  new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Copy16.gif")));
        pasteItem = new JMenuItem("Paste", new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Paste16.gif")));

        cutItem.setEnabled(false); copyItem.setEnabled(false); pasteItem.setEnabled(false);

        JMenu editMenu  = new JMenu("Edit");
        editMenu.add(cutItem); editMenu.add(copyItem); editMenu.add(pasteItem);

        addRectangleItem = new JMenuItem("Add rectangle", new ImageIcon(getClass().getResource("/add_rectangle16.gif")));
        addSquareItem    = new JMenuItem("Add square",    new ImageIcon(getClass().getResource("/add_square16.gif")));

        addRectangleItem.setEnabled(false); addSquareItem.setEnabled(false);

        JMenu polygonsMenu = new JMenu("Model");
        polygonsMenu.add(addRectangleItem); polygonsMenu.add(addSquareItem);

        JMenuBar menuBar = new JMenuBar();
        menuBar.add(fileMenu);
        menuBar.add(editMenu);
        menuBar.add(polygonsMenu);

        setJMenuBar(menuBar);

        refreshControls();
    }

    /**
     * Ritorna la vista associata all'editor.
     *
     * @return La vista associata all'editor.
     */
    public EditorView getView() {
        return view;
    }

    /**
     * Ritorna il modello associato all'editor.
     *
     * @return Il modello associato all'editor.
     */
    public EditorModel getModel() {
        return model;
    }

    /**
     * Aggiunge un rettangolo al modello e rinfresca la vista.
     */
    public void addRectangle() {
        Rectangle     r = new ConcreteRectangle(10, 10, 80, 100);
        RectangleView v = new ColorRectangleView(r);

        model.addViewablePolygon(new ViewableRectangle(v));

        refreshControls();
    }

    /**
     * Aggiunge un quadrato al modello e rinfresca la vista.
     */
    public void addSquare() {
        Square     s = new ConcreteSquare(200, 200, 30);
        SquareView v = new ColorSquareView(s);

        model.addViewablePolygon(new ViewableSquare(v));

        refreshControls();
    }

    /**
     * Rinfresca lo stato (enabled/disabled) dei controlli.
     */
    public void refreshControls() {
        ViewablePolygon selected      = controller.getSelected();
        String          statusBarText = "";

        if(selected != null) {
            Polygon polygon = selected.getPolygon();
            
            statusBarText = polygon.getName() + " - area=" + polygon.area() + ", perimeter=" + polygon.perimeter() + ".";
        } else statusBarText = "No selection.";

        statusBar.setText(statusBarText);
        statusBar.repaint();

        newItem.setEnabled(true);
        openItem.setEnabled(true);
        saveAsItem.setEnabled(true);
        exitItem.setEnabled(true);
        saveItem.setEnabled(model.isModified());

        copyItem.setEnabled(selected != null);

        addRectangleItem.setEnabled(true);
        addSquareItem.setEnabled(true);

        toolBar.getNewButton().setEnabled(true);
        toolBar.getOpenButton().setEnabled(true);
        toolBar.getSaveButton().setEnabled(model.isModified());

        toolBar.getAddRectangleButton().setEnabled(true);
        toolBar.getAddSquareButton().setEnabled(true);

        toolBar.getCopyButton().setEnabled(selected != null);

        toolBar.repaint();

        view.repaint();
    }

    /**
     * Modello dei dati manipolati dall'editor.
     */
    private EditorModel model = null;

    /**
     * Vista dei dati manipolati dall'editor.
     */
    private EditorView view = null;

    /**
     * Controllore dei dati manipolati dall'editor.
     */
    private EditorController controller = null;

    /**
     * Tool bar.
     */
    private EditorToolBar toolBar = null;

    /**
     * Status bar.
     */
    private JLabel statusBar = null;

    /**
     * Item del menu file.
     */
    private JMenuItem newItem = null, openItem = null, saveItem = null, saveAsItem = null, exitItem = null;

    /**
     * Item del menu edit.
     */
    private JMenuItem cutItem = null, copyItem = null, pasteItem = null;

    /**
     * Item del menu model.
     */
    private JMenuItem addRectangleItem = null, addSquareItem = null;
}
