package editor;

import editor.model.*;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToolBar;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * Classe delle toolbar dell'editor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class EditorToolBar extends JToolBar {
    /**
     * Costruttore della toobar dell'editor.
     *
     * @param e L'editor.
     */
    public EditorToolBar(Editor e) {
        super("Editor Tool Bar");

        editor = e;

        Dimension buttonDimension = new Dimension(24, 24);

        newButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/New24.gif")));
        newButton.setToolTipText("New");
        newButton.setPreferredSize(buttonDimension);
        newButton.setMaximumSize(buttonDimension);
        newButton.setEnabled(false);

        openButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Open24.gif")));
        openButton.setToolTipText("Open...");
        openButton.setPreferredSize(buttonDimension);
        openButton.setMaximumSize(buttonDimension);
        openButton.setEnabled(false);

        saveButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Save24.gif")));
        saveButton.setToolTipText("Save");
        saveButton.setPreferredSize(buttonDimension);
        saveButton.setMaximumSize(buttonDimension);
        saveButton.setEnabled(false);

        add(newButton); add(openButton); add(saveButton);

        add(new JToolBar.Separator());

        cutButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Cut24.gif")));
        cutButton.setToolTipText("Cut");
        cutButton.setPreferredSize(buttonDimension);
        cutButton.setMaximumSize(buttonDimension);
        cutButton.setEnabled(false);

        copyButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Copy24.gif")));
        copyButton.setToolTipText("Copy");
        copyButton.setPreferredSize(buttonDimension);
        copyButton.setMaximumSize(buttonDimension);
        copyButton.setEnabled(false);

        pasteButton = new JButton(new ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Paste24.gif")));
        pasteButton.setToolTipText("Paste");
        pasteButton.setPreferredSize(buttonDimension);
        pasteButton.setMaximumSize(buttonDimension);
        pasteButton.setEnabled(false);

        add(cutButton); add(copyButton); add(pasteButton);

        add(new JToolBar.Separator());

        addRectangleButton = new JButton(new ImageIcon(getClass().getResource("/add_rectangle24.gif")));
        addRectangleButton.setToolTipText("Add rectangle");
        addRectangleButton.setPreferredSize(buttonDimension);
        addRectangleButton.setMaximumSize(buttonDimension);
        addRectangleButton.setEnabled(false);

        addSquareButton = new JButton(new ImageIcon(getClass().getResource("/add_square24.gif")));
        addSquareButton.setToolTipText("Add square");
        addSquareButton.setPreferredSize(buttonDimension);
        addSquareButton.setMaximumSize(buttonDimension);
        addSquareButton.setEnabled(false);

        add(addRectangleButton); add(addSquareButton);

        setRollover(true);

        addRectangleButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    editor.addRectangle();
                }
            }
        );

        addSquareButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    editor.addSquare();
                }
            }
        );

        newButton.addActionListener(
            new ActionListener() {
                public void actionPerformed(ActionEvent e) {   
                    editor.newFile();
                }
            }
        );
    }

    /**
     * Rinfresca lo stato (enabled/disabled) dei controlli.
     */
    public void refreshControls() {
        ViewablePolygon selected = editor.getController().getSelected();

        newButton.setEnabled(true);
        openButton.setEnabled(true);
        saveButton.setEnabled(editor.getModel().isModified());

        addRectangleButton.setEnabled(true);
        addSquareButton.setEnabled(true);

        copyButton.setEnabled(selected != null);

        repaint();
    }

    /**
     * Ritorna il bottone di new file.
     *
     * @return Il bottone di new file.
     */
    public JButton getNewButton() {
        return newButton;
    }

    /**
     * Ritorna il bottone di open file.
     *
     * @return Il bottone di open file.
     */
    public JButton getOpenButton() {
        return openButton;
    }

    /**
     * Ritorna il bottone di save file.
     *
     * @return Il bottone di save file.
     */
    public JButton getSaveButton() {
        return saveButton;
    }

    /**
     * Ritorna il bottone di copia nella clipboard.
     *
     * @return Il bottone di copia nella clipboard.
     */
    public JButton getCopyButton() {
        return copyButton;
    }

    /**
     * Ritorna il bottone di taglia.
     *
     * @return Il bottone di taglia.
     */
    public JButton getCutButton() {
        return cutButton;
    }

    /**
     * Ritorna il bottone di incolla dalla clipboard.
     *
     * @return Il bottone di incolla dalla clipboard.
     */
    public JButton getPasteButton() {
        return pasteButton;
    }

    /**
     * Ritorna il bottone di aggiungi rettangolo.
     *
     * @return Il bottone di aggiungi rettangolo.
     */
    public JButton getAddRectangleButton() {
        return addRectangleButton;
    }

    /**
     * Ritorna il bottone di aggiungi quadrato.
     *
     * @return Il bottone di aggiungi quadrato.
     */
    public JButton getAddSquareButton() {
        return addSquareButton;
    }

    /**
     * L'editor.
     */
    private Editor editor = null;

    /**
     * Bottoni di controllo dei file.
     */
    private JButton newButton = null, openButton = null, saveButton = null;

    /**
     * Bottoni di controllo editor.
     */
    private JButton cutButton = null, copyButton = null, pasteButton = null;

    /**
     * Bottoni di controllo del modello.
     */
    private JButton addRectangleButton = null, addSquareButton = null;
}
