package editor.view;

import editor.model.*;

import javax.swing.JPanel;

import java.awt.Graphics;

/**
 * Classe delle viste dell'editor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class EditorView extends JPanel {
    /**
     * Costruttore del modello dell'editor.
     *
     * @param m Il modello dell'editor.
     */
    public EditorView(EditorModel m) {
        model = m;
    }

    /**
     * Metoto utilizzato per disegnare il contenuto del JPanel.
     *
     * @param g Il contesto grafico in cui disegnare.
     */
    public void paintComponent(Graphics g) {
        if(isDragging) {
            Polygon s = selected.getPolygon();

            g.setXORMode(getBackground());

            if(lastX != -1 && lastY != -1) 
                g.drawRect(lastX,        lastY, 
                           s.getWidth(), s.getHeight());

            lastX = s.getX(); lastY = s.getY();

            g.drawRect(lastX,        lastY, 
                       s.getWidth(), s.getHeight());

            return;
        }

        super.paintComponent(g);

        for(int i = 0; i < model.getSize(); i++) {
            ViewablePolygon p = model.getViewablePolygon(i);
            PolygonView     v = p.getPolygonView();

            v.paintPolygon(g);
        }
    }

    /**
     * Imposta lo stato di dragging del controller.
     *
     * @param b Lo stato di dragging del controller.
     */
    public void setDragging(boolean b) {
        isDragging = b;

        if(!isDragging) { lastX = -1; lastY = -1; }
    }

    /**
     * Imposta il poligono correntemente selezionato.
     *
     * @param s Il poligono correntemente selezionato.
     */
    public void setSelected(ViewablePolygon s) {
        selected = s;
    }

    /**
     * Modello da visualizzare.
     */
    private EditorModel model = null;

    /**
     * Poligono correntemente selezionato.
     */
    private ViewablePolygon selected = null;

    /**
     * Controller associato alla vista.
     */
    private boolean isDragging = false;

    /**
     * Ultima coordinata X del puntatore del mouse durante il dragging.
     */
    private int lastX = -1;

    /**
     * Ultima coordinata Y del puntatore del mouse durante il dragging.
     */
    private int lastY = -1;
}
