package editor.model;

/**
 * Classe dei modelli utilizzati dall'editor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class EditorModel {
    /**
     * Costruttore del modello dell'editor.
     */
    public EditorModel() {
        polygons = new ViewablePolygon[100];
        size     = 0;
    }

    /**
     * Aggiunge un poligono al modello.
     *
     * @param p Il poligono da aggiungere.
     */
    public void addViewablePolygon(ViewablePolygon p) {
        polygons[size++] = p;

        setModified(true);
    }

    /**
     * Ritorna un poligono del modello.
     *
     * @param i Indice del poligono da ritornare.
     */
    public ViewablePolygon getViewablePolygon(int i) {
        return polygons[i];
    }

    /**
     * Ritorna la lunghezza attuale della lista.
     *
     * @return Lunghezza della lista.
     */
    public int getSize() {
        return size;
    }

    /**
     * Imposta lo stato di modifica del modello.
     *
     * @param m Lo stato di modifica del modello.
     */
    public void setModified(boolean m) {
        modified = m;
    }

    /**
     * Ritorna lo stato di modifica del modello.
     *
     * @return Lo stato di modifica del modello.
     */
    public boolean isModified() {
        return modified;
    }

    /**
     * Ritorna lo stato di modifica del modello.
     *
     * @return Lo stato di modifica del modello.
     */
    public boolean getModified() {
        return modified;
    }

    /**
     * Ritorna il nome del file del modello.
     *
     * @return Il nome del file del modello.
     */
    public String getFileName() {
        return fileName;
    }

    /**
     * Imposta il nome del file del modello.
     *
     * @param n Il nome del file del modello.
     */
    public void setFileName(String n) {
        fileName = n;
    }

    /**
     * Nome del file del modello.
     */
    private String fileName = null;

    /**
     * Indica se il modello e' stato modificato.
     */
    private boolean modified = false;

    /**
     * Lunghezza attuale della lista dei poligoni.
     */
    private int size = 0;

    /**
     * Lista che contiene i poligoni del modello.
     */
    private ViewablePolygon[] polygons = null;
}
