package login;

import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.util.*;
import java.util.List;

public class JLoginPanel extends JPanel {
    public static final String OK_STRING               = "okString",       CANCEL_STRING          = "cancelString",
                               INSERT_USER_NAME_STRING = "insertUserName", INSERT_PASSWORD_STRING = "insertUserName",
                               ICON                    = "icon";

    public JLoginPanel() {
        model = new AbstractLoginModel("", 3, false) {
            public boolean validate(String userName, String password) {
                return false;
            }
        };

        center        = new JPanel();
        bottom        = new JPanel();
        iconContainer = new JPanel();

        setLayout(new BorderLayout());

        add(iconContainer, BorderLayout.CENTER);
        add(bottom,        BorderLayout.SOUTH);

        ok     = new JButton(okString);
        cancel = new JButton(cancelString);

        bottom.setLayout(new FlowLayout());
        bottom.add(ok);
        bottom.add(cancel);

        ok.setEnabled(model.isEmptyPasswordAllowed());

        nameField           = new JTextField(model.getInitialUserName());
        passwordField       = new JPasswordField();
        insertUserNameLabel = new JLabel(insertUserNameString);
        insertPasswordLabel = new JLabel(insertPasswordString);
        iconLabel           = new JLabel();

        JPanel namePasswordPanel = new JPanel();
        namePasswordPanel.setLayout(new GridLayout(4, 1));
        namePasswordPanel.add(insertUserNameLabel);
        namePasswordPanel.add(nameField);
        namePasswordPanel.add(insertPasswordLabel);
        namePasswordPanel.add(passwordField);

        center.setLayout(new BorderLayout());
        center.add(namePasswordPanel, BorderLayout.NORTH);

        iconContainer.setLayout(new BorderLayout());
        iconContainer.add(center,    BorderLayout.CENTER);
        iconContainer.add(iconLabel, BorderLayout.WEST);

        ok.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                String userName = nameField.getText(),
                       password = passwordField.getText();

                if(model.validate(userName, password)) fireLoginValidated(new LoginEvent(JLoginPanel.this, LoginEvent.OK));

                repetitionsCount++;

                if(repetitionsCount == model.getAllowedRepetitions()) fireLoginFailed(new LoginEvent(JLoginPanel.this, LoginEvent.OK));
                else                                                  passwordField.setText("");
            }
        });

        cancel.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent event) {
                fireLoginCancelled(new LoginEvent(JLoginPanel.this, LoginEvent.CANCEL));
            }
        });

        DocumentListener documentListener = new DocumentListener() {
            public void changedUpdate(DocumentEvent e) {
                checkEmptyFields();
            }

            public void insertUpdate(DocumentEvent e) {
                checkEmptyFields();
            }

            public void removeUpdate(DocumentEvent e) {
                checkEmptyFields();
            }
        };

        nameField.getDocument().addDocumentListener(documentListener);
        passwordField.getDocument().addDocumentListener(documentListener);
    }

    public void setModel(LoginModel m) {
        model = m;

        reset();
    }

    public void setOKString(String s) {
        String old = okString;

        okString = s; ok.setLabel(okString);

        firePropertyChange(OK_STRING, old, okString);
    }

    public String getOKString() {
        return okString;
    }

    public String getUserName() {
        return nameField.getText();
    }

    public void setCancelString(String s) {
        String old = cancelString;

        cancelString = s; cancel.setLabel(cancelString);

        firePropertyChange(CANCEL_STRING, old, cancelString);
    }

    public String getCancelString() {
        return cancelString;
    }

    public void setInsertUserNameString(String s) {
        String old = insertUserNameString;

        insertUserNameString = s; insertUserNameLabel.setText(insertUserNameString);

        firePropertyChange(INSERT_USER_NAME_STRING, old, insertUserNameString);
    }

    public String getInsertUserNameString() {
        return insertUserNameString;
    }

    public void setInsertPasswordString(String s) {
        String old = insertPasswordString;

        insertPasswordString = s; insertPasswordLabel.setText(insertPasswordString);

        firePropertyChange(INSERT_PASSWORD_STRING, old, insertPasswordString);
    }

    public String getInsertPasswordString() {
        return insertPasswordString;
    }

    public void setIcon(Icon i) {
        Icon old = icon;

        icon = i; iconLabel.setIcon(icon);

        firePropertyChange(ICON, old, icon);
    }

    public Icon getIcon() {
        return icon;
    }
    
    public void addLoginListener(LoginListener l) {
        loginListeners.add(l);
    }

    public void removeLoginListener(LoginListener l) {
        loginListeners.remove(l);
    }

    protected void fireLoginValidated(LoginEvent e) {
        reset();

        for(int i = 0; i < loginListeners.size(); i++) {
            Object o = loginListeners.get(i);
        	
            ((LoginListener)o).loginValidated(e);
        }
    }

    protected void checkEmptyFields() {
        if(nameField.getText().length() == 0) {
            ok.setEnabled(false); return;
        }

        if(!model.isEmptyPasswordAllowed() && passwordField.getText().length() == 0) {
            ok.setEnabled(false); return;
        }

        ok.setEnabled(true);
    }

    protected void fireLoginFailed(LoginEvent e) {
        reset();

        for(int i = 0; i < loginListeners.size(); i++) {
            Object o = loginListeners.get(i);
        	
            ((LoginListener)o).loginFailed(e);
        }
    }

    protected void fireLoginCancelled(LoginEvent e) {
        reset();

        for(int i = 0; i < loginListeners.size(); i++) {
            Object o = loginListeners.get(i);
        	
            ((LoginListener)o).loginCancelled(e);
        }
    }

    protected void reset() {
        repetitionsCount = 0;

        nameField.setText(model.getInitialUserName());
        passwordField.setText("");
    }

    protected LoginModel model;
    protected int        repetitionsCount = 0;
    
    protected JPanel     bottom,              center,
                         iconContainer;
    protected JButton    ok,                  cancel;
    protected JTextField nameField,           passwordField;
    protected JLabel     insertUserNameLabel, insertPasswordLabel,
                         iconLabel;

    protected Icon   icon;
    protected String okString             = "OK",
                     cancelString         = "Annulla",
                     insertUserNameString = "Introdurre il nome",
                     insertPasswordString = "Introdurre la password";

    protected List loginListeners = new ArrayList();
}
