package textprocessor;

import java.io.FileInputStream;
import java.io.InputStream;
import java.io.IOException;

import java.util.ArrayList;
import java.util.List;

/**
 * Classe dei text processor.
 *
 * @author  Federico Bergenti
 * @version 1.00 10/06/03
 */
public class TextProcessor {
    /**
     * Crea un nuovo text processor.
     */
    public TextProcessor() {
        listeners = new ArrayList();
    }

    /**
     * Processa un file passato come parametro.
     *
     * @param fileName Il nome del file da processare.
     * @throw TextProcessorException
     */    
    public void process(String fileName) throws TextProcessorException {
        try {
            InputStream is = new FileInputStream(fileName);
            
            process(is);
        } catch(TextProcessorException tpe) {
            throw tpe;
        } catch(IOException ioe) {
            throw new TextProcessorException("Cannot open input file");
        }
    }

    /**
     * Processa il contenuto di un <code>InputStream</code> passato come parametro.
     *
     * @param is L'<code>InputStream</code>da processare.
     * @throw TextProcessorException
     */    
    public void process(InputStream is) throws TextProcessorException {
        String word = "", paragraph = "";
        int    c    = 0;
        
        try {
            while((c = is.read()) != -1) {
                char ch = (char)c;
            
                if(ch == ' ') { 
                    notifyWord(word); word = "";
                } else {
                    notifyCharacter(ch); word += ch;
                }
            }

            notifyWord(word);
            
            notifyEOF();
        } catch(IOException ioe) {
            throw new TextProcessorException("Problems reading data from input stream");
        }
    }

    /**
     * Aggiunge un listener.
     *
     * @param tpl Il listener da aggiungere.
     */        
    public void addListener(TextProcessorListener tpl) {
        listeners.add(tpl);
    }

    /**
     * Rimuove un listener.
     *
     * @param tpl Il listener da rimuovere.
     */
    public void removeListener(TextProcessorListener tpl) {
        listeners.remove(tpl);
    }

    /**
     * Notifica i listener che e' stato letto un nuovo carattere.
     *
     * @param c Il carattere letto.
     */        
    private void notifyCharacter(char c) {
        for(int i = 0; i < listeners.size(); i++) {
            TextProcessorListener tl = (TextProcessorListener)listeners.get(i);
            
            tl.characterRead(c);
        }
    }

    /**
     * Notifica i listener che e' stata letta una nuova parola.
     *
     * @param w La parola letta.
     */        
    private void notifyWord(String w) {
        if(w == null || w.length() == 0) return;

        for(int i = 0; i < listeners.size(); i++) {
            TextProcessorListener tl = (TextProcessorListener)listeners.get(i);
            
            tl.wordRead(w);
        }
    }

    /**
     * Notifica i listener che e' stata raggiunta la fine del file.
     */        
    private void notifyEOF() {
        for(int i = 0; i < listeners.size(); i++) {
            TextProcessorListener tl = (TextProcessorListener)listeners.get(i);
            
            tl.eofReached();
        }
    }

    /**
     * Lista dei listener.
     */            
    private List listeners = null;
}